<?php
namespace Teknisa\TCO\Authentication;

use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class TcoAuthListener extends  \Zeedhi\Framework\Events\PreDispatch\Listener {

    /**
     * @var array $routesParams 
     *  Array that contains routes
     *  as keys and, as value, an array with
     *  authentication methods names as key
     *  and the errorcode as value. The method
     *  should be implemented at
     *  'TCO\Authentication\TcoAuthentication'.
     */
    protected $routesParams;
    protected $tcoAuthenticator;

    /**
     * TcoAuthListener constructor.
     * @param array $routesParams
     */
    public function __construct($routesParams, $tcoAuthenticator) {
        $this->routesParams     = $routesParams;
        $this->tcoAuthenticator  = $tcoAuthenticator;
    }

    /**
     * Listen to request, then call TcoAuthentication
     * methods if the current route is configured
     * in 'services.xml'.
     *
     * @param Request $request
     */
    public function preDispatch(Request $request) {
        $currentRoute = $request->getRoutePath();
        if(isset($this->routesParams[$currentRoute])){
            $this->callAuthenticator($currentRoute, $request);
        }
    }

    /**
     * Call authenticator with current route params
     * If TcoAuthentication::method return false 
     * for a invalid authentication, it will throw 
     * a new \Exception with the error code.
     * 
     * @param string $currentRoute
     * @param object $request
     */
    private function callAuthenticator($currentRoute, $request){
        foreach($this->routesParams[$currentRoute] as $method => $errorCode){
            if(!$this->tcoAuthenticator->run($method, $request)){
                throw Exception::userMadeError($errorCode, -1);
            }
        }
    }
}