<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;

class Tco00100RamoAtividadeSegmento extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }

    public function salvaRamosAtividade(DTO\Request $request, DTO\Response $response) {
        $ramosAtividade = $request->getDataSet()->getRows();
        $ramosExistentes = $this->retornaRamosAtividadeVinculadosExistentes($ramosAtividade);
        
        if(empty($ramosExistentes)){
            $this->salvaRamosAtividadeSegmento($ramosAtividade);
            $response->addDataSet(new DataSet("ramosAtividade", array()));
        } else {
            $response->addDataSet(new DataSet("ramosAtividade", $ramosExistentes));
        }
    }
    
    private function retornaRamosAtividadeVinculadosExistentes($ramosAtividade) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT 'TCO00100_RAMOS_ATIVIDADE_SEGMENTO_EXISTENTES' AS MESSAGECODE,
                       ATIVCLISEGMER.NRATIVCLISEGMER,
                       ATIVCLISEGMER.CDATIVCLI,
                       ATIVCLISEGMER.CDSEGMER,
                       SEGMERCADO.DSSEGMER,
                       ATIVCLIENTE.DSATIVCLI,
                       ATIVCLISEGMER.NRORG
                  FROM ATIVCLISEGMER
                 INNER JOIN ATIVCLIENTE 
                    ON ATIVCLIENTE.CDATIVCLI = ATIVCLISEGMER.CDATIVCLI
                 INNER JOIN SEGMERCADO
                    ON SEGMERCADO.CDSEGMER = ATIVCLISEGMER.CDSEGMER
                 WHERE (INSTR(:CDATIVCLICDSEGMER, '_'||ATIVCLISEGMER.CDATIVCLI||'-'||ATIVCLISEGMER.CDSEGMER) > 0)
                   AND ATIVCLIENTE.NRORG = :NRORG ";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "CDATIVCLICDSEGMER"  => $this->concatAtivcliSegmer($ramosAtividade),
            "NRORG"              => $ramosAtividade[0]["NRORG"]
        ); 
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function concatAtivcliSegmer($ramosAtividade) {
        $cdativclicdsegmer = "";
        
        foreach ($ramosAtividade as $ramo) {
            $cdativclicdsegmer .= "_".$ramo['CDATIVCLI']."-".$ramo['CDSEGMER'];
        }
        
        return $cdativclicdsegmer;
    }
    
    private function salvaRamosAtividadeSegmento($ramosAtividade) {
        $newdataSet = new DataSet("ativclisegmer", $ramosAtividade);
        $newrequest = new DTO\Request\DataSet($newdataSet, "", "", "");
        $rows = $this->crudService->getRowsToSave($newrequest, "ativclisegmer");
        $this->crudService->save("ativclisegmer", $rows);
    }
    
    public function removeRamosAtividade(DTO\Request $request, DTO\Response $response) {
        $ramosAtividadeSegmento = $request->getDataSet()->getRows();
        $ramosAtividadeUtilizados = $this->verificaRamosAtividadeUtilizados($ramosAtividadeSegmento); 
        
        if(empty($ramosAtividadeUtilizados)){
            $this->removeRamosAtividadeSegmento($request, $response);
            $response->addDataSet(new DataSet("ramosAtividadeSegmentoUtilizados", array()));
        } else {
            $response->addDataSet(new DataSet("ramosAtividadeSegmentoUtilizados", $ramosAtividadeUtilizados));
        }
        
    }
    
    private function verificaRamosAtividadeUtilizados($ramosAtividadeSegmento) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT MESSAGECODE,
                       NRATIVCLISEGMER,
                       CDATIVCLI,
                       CDSEGMER,
                       DSSEGMER,
                       DSATIVCLI,
                       CDADESAOCARDBASE,
                       CDCARDAPIOBASE,
                       NRORG
                  FROM ( SELECT 'TCO00100_RAMOS_ATIVIDADE_SEGMENTO_UTILIZADOS_ADESAOCARDBASE' AS MESSAGECODE,
                                ATIVCLISEGMER.NRATIVCLISEGMER,
                                ATIVCLISEGMER.CDATIVCLI,
                                ATIVCLISEGMER.CDSEGMER,
                                SEGMERCADO.DSSEGMER,
                                ATIVCLIENTE.DSATIVCLI,
                                ADESAOCARDBASE.CDADESAOCARDBASE,
                                CARDAPIOBASE.CDCARDAPIOBASE,
                                ATIVCLISEGMER.NRORG
                           FROM ATIVCLISEGMER
                          INNER JOIN ATIVCLIENTE 
                             ON ATIVCLIENTE.CDATIVCLI = ATIVCLISEGMER.CDATIVCLI
                          INNER JOIN SEGMERCADO
                             ON SEGMERCADO.CDSEGMER = ATIVCLISEGMER.CDSEGMER
                          INNER JOIN ATIVCLIADESAOCBASE
                             ON ATIVCLISEGMER.NRATIVCLISEGMER = ATIVCLIADESAOCBASE.NRATIVCLISEGMER
                          INNER JOIN ADESAOCARDBASE
                             ON ADESAOCARDBASE.NRADESAOCARDBASE = ATIVCLIADESAOCBASE.NRADESAOCARDBASE
                          INNER JOIN CARDAPIOBASE
                             ON CARDAPIOBASE.NRCARDAPIOBASE = ADESAOCARDBASE.NRCARDAPIOBASE
                         
                          UNION ALL
                         
                         SELECT 'TCO00100_RAMOS_ATIVIDADE_SEGMENTO_UTILIZADOS_PARTSEGCARDBASE' AS MESSAGECODE,
                                ATIVCLISEGMER.NRATIVCLISEGMER,
                                ATIVCLISEGMER.CDATIVCLI,
                                ATIVCLISEGMER.CDSEGMER,
                                SEGMERCADO.DSSEGMER,
                                ATIVCLIENTE.DSATIVCLI,
                                NULL AS CDADESAOCARDBASE,
                                CARDAPIOBASE.CDCARDAPIOBASE,
                                ATIVCLISEGMER.NRORG
                           FROM ATIVCLISEGMER
                          INNER JOIN ATIVCLIENTE 
                             ON ATIVCLIENTE.CDATIVCLI = ATIVCLISEGMER.CDATIVCLI
                          INNER JOIN SEGMERCADO
                             ON SEGMERCADO.CDSEGMER = ATIVCLISEGMER.CDSEGMER
                          INNER JOIN PARTSEGCARDBASE
                             ON ATIVCLISEGMER.NRATIVCLISEGMER = PARTSEGCARDBASE.NRATIVCLISEGMER 
                          INNER JOIN CARDAPIOBASE
                             ON CARDAPIOBASE.NRCARDAPIOBASE = PARTSEGCARDBASE.NRCARDAPIOBASE ) 
                 WHERE (INSTR(:CDATIVCLICDSEGMER, '_'||CDATIVCLI||'-'||CDSEGMER) > 0)
                   AND NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "CDATIVCLICDSEGMER"  => $this->concatAtivcliSegmer($ramosAtividadeSegmento),
            "NRORG"              => $ramosAtividadeSegmento[0]["NRORG"]
        ); 
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function removeRamosAtividadeSegmento($request, $response) {
		$rows = $this->crudService->getRowsToDelete($request, 'ativclisegmer');
        $dataSet = new DataSet("ativclisegmer", $rows);
		$this->crudService->delete($response, $dataSet);
	}
	
	public function mudarEstado($request, $response) {
	    $this->entityManager->beginTransaction();
	    $rows = $request->getDataSet()->getRows();
	    
	    try{
            $connection = $this->entityManager->getConnection();
            $sql = "UPDATE SEGMERCADO 
                       SET IDHABILITADOTCO = :IDHABILITADOTCO 
                     WHERE CDSEGMER = :CDSEGMER
                       AND NRORG = :NRORG ";
            
            $query = $connection->prepare($sql);
            foreach ($rows as $row) {
                $params = array(
                    "IDHABILITADOTCO"  => $row['IDHABILITADOTCO'],
                    "CDSEGMER"         => $row['CDSEGMER'],
                    "NRORG"            => $this->environment->getNrOrgTrab()
                ); 
                $query->execute($params);
            }
            
            $this->entityManager->commit();
	    }catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
	    }
	}

}
